package biga 
{
	import flash.geom.Point;
	/**
	 * @author Nicolas Barradeau
	 * http://en.nicoptere.net
	 */
	public class GeomUtils
	{
		
		/**
		 * retourne l'angle entre p0 et p1
		 * @param	p0
		 * @param	p1
		 * @return
		 */
		static public function angle( p0:Point, p1:Point ):Number
		{
			
			return Math.atan2( p1.y - p0.y, p1.x - p0.x );
			
		}
		
		/**
		 * retourne la pente entre p0 et p1
		 * @param	p0
		 * @param	p1
		 * @return
		 */
		static public function slope( p0:Point, p1:Point ):Number
		{
			
			return ( p1.y - p0.y ) / ( p1.x - p0.x );
			
		}
		
		/**
		 * retourne la distance entre p0 et p1
		 * @param	p0
		 * @param	p1
		 * @return
		 */
		static public function distance( p0:Point, p1:Point ):Number
		{
			
			return Math.sqrt( squareDistance( p0.x, p0.y, p1.x, p1.y ) );
			
		}
		
		/**
		 * retourne la distance entre p0 et p1
		 * @param	p0
		 * @param	p1
		 * @return
		 */
		static public function squareDistance( x0:Number, y0:Number, x1:Number, y1:Number ):Number
		{
			
			return ( x0 - x1 ) * ( x0 - x1 ) + ( y0 - y1 ) * ( y0 - y1 );
			
		}
		
		/**
		 * retourne le palier dans lequel la valeur se situe
		 * @param	value un Number normalise (0-1)
		 * @param	step valeur du palier
		 * @return
		 */
		static public function toFixed( value:Number, step:Number = 10 ):Number
		{
			return int( ( value * step ) ) / step ;
		}
		
		/**
		 * retourne le nombre de fois que step tient dans value sans faire un chiffre à virgule
		 * @param	value 
		 * @param	step valeur du palier
		 * @return
		 */
		static public function snap( value:Number, step:Number ):Number
		{
			return int( value / step ) * step;
		}
		
		/**
		 * borne une valeur entre 2 valeurs
		 * @param	value
		 * @param	min
		 * @param	max
		 * @return
		 */
		static public function clamp( value:Number, min:Number, max:Number ):Number
		{
			return value < min ? min : ( value > max ? max : value );
		}
		
		/**
		 * interpolation lineaire de T entre 2 valeurs A et B
		 * @param	t
		 * @param	a
		 * @param	b
		 * @return
		 */
		static public function lerp( t:Number, a:Number, b:Number ):Number
		{
			
			var min:Number = ( a < b ) ? a : b;
			var max:Number = ( a > b ) ? a : b;
			return min + ( max - min ) * t;
			
		}
		
		/**
		 * retourne la valeur normalisée de value entre A et B
		 * @param	value
		 * @param	a
		 * @param	b
		 * @return
		 */
		static public function normalize( value:Number, a:Number, b:Number ):Number
		{
			
			var min:Number = ( a < b ) ? a : b;
			var max:Number = ( a > b ) ? a : b;
			return (value - min) / (max - min);
			
		}
		
		/**
		 * permet de borner une valeur entre 2 plages de valeurs 
		 * @param	value
		 * @param	min1
		 * @param	max1
		 * @param	min2
		 * @param	max2
		 * @return
		 */
		static public function map(value:Number, min1:Number, max1:Number, min2:Number, max2:Number):Number
		{
			
			return lerp( normalize( value, min1, max1 ), min2, max2 );
			
		}
		
		/**************************************************************
		
				VECTOR UTILS
		
		**************************************************************/
		
		static public function determinant( p:Point, a:Point, b:Point ):Number
		{
			return ( ( a.x - b.x ) * ( p.y - b.y ) ) - ( ( p.x - b.x ) * ( a.y - b.y ) );
		}
		
		static public function isLeft( p:Point, a:Point, b:Point ):Boolean
		{
			return determinant( p, a, b ) >= 0;
		}
		
		static public function isRightt( p:Point, a:Point, b:Point ):Boolean
		{
			return determinant( p, a, b ) <= 0;
		}
		
		public function normalizeVector( p:Point ):Point
		{
			
			return new Point( p.x / p.length, p.y / p.length );
			
		}
		
		public function dotProduct( p0:Point, p1:Point ):Number
		{
			
			var u:Point = normalizeVector( p0 );
			var v:Point = normalizeVector( p1 );
			return ( u.x * v.x + u.y * v.y );
			
		}
		
		public function crossProduct( p0:Point, p1:Point ):Number
		{
			
			var u:Point = normalizeVector( p0 );
			var v:Point = normalizeVector( p1 );
			return ( u.x * v.y - u.y * v.x );
			
		}
		
		
		// fast & furious projection method
		// http://www.vcskicks.com/code-snippet/point-projection.php
		static public function project( p:Point, a:Point, b:Point ):Point
		{
			
			var m:Number = ( b.y - a.y ) / ( b.x - a.x );
			var m2:Number = m * m;
			var i:Number = a.y - ( m * a.x );
			
			return new Point( 	( m * p.y + p.x - m * i)  / ( m2 + 1), 
								( m2 * p.y + m * p.x + i) / ( m2 + 1)	 );
			
		}
		/*---------------------------------------------------------------------------
		http://keith-hair.net/blog/
		
		Returns the closest Point on Line "AB" to Point "p":
		p			-Point, the Point to constrain to the line.
		A			-Point, beginning Point of Line.
		B			-Point, ending Point of Line.
		as_seg	-Boolean, limits return Point between endpoints of A and B.
		----------------------------------------------------------------------------*/
		static public function constrain( p:Point, a:Point, b:Point ):Point
		{
			
			var dx:Number = b.x - a.x;
			var dy:Number = b.y - a.y;
			
			if (dx == 0 && dy == 0) 
			{
				return a;
			}
			else
			{
				var t:Number = ( (p.x - a.x) * dx + (p.y - a.y) * dy) / (dx * dx + dy * dy);
				
				t = Math.min( Math.max( 0, t ), 1 );
				
				return new Point(	a.x + t * dx, a.y + t * dy	);
				
			}
			
			return null;
			
		}
		
	}
}