package biga 
{
	import flash.geom.Point;
	import biga.Edge;
	import biga.Intersection;
	import biga.Triangle;
	import biga.GeomUtils;
	
	/**
	 * @author Nicolas Barradeau
	 * http://en.nicoptere.net
	 */
	public class TriangleUtils 
	{
		
		public function TriangleUtils(){}
		
		static public function bissector( triangle:Triangle, id:int ):Edge
		{
			
			var e:Edge;
			var angle:Number;
			var p0:Point, p1:Point;
			
			switch( id )
			{
				case 0:
				p0 = triangle.p0;
				angle = ( GeomUtils.angle( p0, triangle.p2 ) + GeomUtils.angle( p0, triangle.p1 ) ) * .5;
				break;
				
				case 1: 
				p0 = triangle.p1;
				angle = ( GeomUtils.angle( p0, triangle.p0 ) + GeomUtils.angle( p0, triangle.p2 ) ) * .5;
				break;
				
				case 2: 
				p0 = triangle.p2;
				angle = ( GeomUtils.angle( p0, triangle.p1 ) + GeomUtils.angle( p0, triangle.p0 ) ) * .5;
				break;
				
			}
			
			e = triangle.edges[ ( id + 1 ) % 3 ];//opposite side
			
			p1 = new Point( p0.x + Math.cos( angle ),
							p0.y + Math.sin( angle )	);
			
			return new Edge( p0, Intersection.lineIntersectLine( p0, p1, e.p0, e.p1, false, false ) );
			
		}
		
		static public function incircle( triangle:Triangle ):Circle
		{
			
			var a:Edge = bissector( triangle, 0 );
			var b:Edge = bissector( triangle, 1 );
			var c:Edge = bissector( triangle, 2 );
			
			var ip:Point = Intersection.lineIntersectLine( a.p0, a.p1, b.p0, b.p1 );
			
			var p0:Point = GeomUtils.project( ip, triangle.p0, triangle.p1 );
			var p1:Point = GeomUtils.project( ip, triangle.p1, triangle.p2 );
			var p2:Point = GeomUtils.project( ip, triangle.p2, triangle.p0 );
			
			var r0:Number = GeomUtils.distance( ip, p0 );
			var r1:Number = GeomUtils.distance( ip, p1 );
			var r2:Number = GeomUtils.distance( ip, p2 );
			
			return new Circle( ip.x, ip.y, Math.min( r0, r1, r2 ) );
			
		}
		
		
		static public function median( triangle:Triangle, id:int ):Edge
		{
			
			switch( id )
			{
				case 0: return new Edge( triangle.p0, triangle.edges[1].center );
				case 1: return new Edge( triangle.p1, triangle.edges[2].center );
				case 2: return new Edge( triangle.p2, triangle.edges[0].center );
			}
			return null;
			
		}
		
		static public function centroid( triangle:Triangle ):Point
		{
			return new Point( 	( triangle.p0.x + triangle.p1.x + triangle.p2.x ) * 1 / 3, 
								( triangle.p0.y + triangle.p1.y + triangle.p2.y ) * 1 / 3 );
			
		}
		// =
		static public function gravityCenter( triangle:Triangle ):Point
		{
			
			var a:Edge = median( triangle, 0 );
			var b:Edge = median( triangle, 1 );
			return Intersection.lineIntersectLine( a.p0, a.p1, b.p0, b.p1, false, false );
			
		}
		
		static public function altitude( triangle:Triangle, id:int ):Edge
		{
			
			switch( id )
			{
				case 0: return new Edge( triangle.p0, GeomUtils.project( triangle.p0, triangle.p1, triangle.p2 )  );
				case 1: return new Edge( triangle.p1, GeomUtils.project( triangle.p1, triangle.p0, triangle.p2 )  );
				case 2: return new Edge( triangle.p2, GeomUtils.project( triangle.p2, triangle.p0, triangle.p1 )  );
			}
			return null;
			
		}
		
		static public function orthocenter(triangle:Triangle):Point 
		{
			
			var a:Edge = altitude( triangle, 0 );
			var b:Edge = altitude( triangle, 1 );
			return Intersection.lineIntersectLine( a.p0, a.p1, b.p0, b.p1, false, false );
			
		}
		
		static public function circumcircle( triangle:Triangle ):Circle 
		{
			
			var e0:Edge = new Edge( triangle.edges[ 0 ].center,  triangle.p1 ).leftNormal;
			var e1:Edge = new Edge( triangle.edges[ 1 ].center,  triangle.p2 ).leftNormal;
			var ce:Point =  Intersection.lineIntersectLine( e0.p0, e0.p1, e1.p0, e1.p1, false, false );
			
			if( ce != null ) return new Circle( ce.x, ce.y, GeomUtils.distance( ce, triangle.p0 ) );
			return null;
			
		}
		
		static public function eulerLine( triangle:Triangle ):Edge
		{
			
			var c:Circle = circumcircle( triangle );
			if ( c == null ) return null;
			var a:Point = new Point( c.x, c.y );
			var b:Point = orthocenter( triangle );
			
			return new Edge( a, b );
			
		}
		
		static public function eulerCircle( triangle:Triangle ):Circle 
		{
			
			var a:Point = altitude( triangle, 0 ).p1;
			var b:Point = altitude( triangle, 1 ).p1;
			var c:Point = altitude( triangle, 2 ).p1;
			
			return circumcircle( new Triangle( a, b, c ) );
			
		}
		
	}

}